==> subfind_provider_subscene-3.1.1.0-py2.py3-none-any.whl: subfind_provider_subscene/__init__.py <==
import logging
import zipfile

from six.moves.urllib.parse import urlencode, urljoin

import os
import re
import requests
from os import listdir
from os.path import join, abspath
from shutil import rmtree
from subfind import BaseProvider
from subfind.cmd import run_cmd
from subfind.exception import SubtitleFileBroken, HTTPConnectionError, ReleaseNotMatchError
from subfind.parser import Parser
from subfind.release import ReleaseMatchingChecker
from subfind.release.alice import ReleaseScoringAlice
from subfind.scenario import BaseScenarioFactory, Scenario1
from subfind.utils.subtitle import get_subtitle_ext
from subfind_provider_subscene.language import get_full_lang, get_short_lang
from tempfile import mkdtemp

SUBSCENE_SEARCH_URL = "http://subscene.com/subtitles/title?%s"
SUBSCENE_RELEASE_SEARCH_URL = "http://subscene.com/subtitles/release?%s"


class SubsceneFactory(BaseScenarioFactory):
    def get_scenario(self):
        release_scoring = ReleaseScoringAlice()
        provider = SubsceneProvider()

        return Scenario1(release_scoring, provider)


class SubsceneProvider(BaseProvider):
    def __init__(self):
        self.session = requests.Session()

        self.movie_title_year_pattern = re.compile('^(.*)(\s+\((\d+)\))$')
        self.sub_extensions = ['srt', 'sub', 'ass']
        self.logger = logging.getLogger(self.__class__.__name__)

    def search_movie(self, params):
        query = params['title_query']
        base_url = (SUBSCENE_SEARCH_URL % urlencode({'q': query}))

        r = self.session.get(base_url)

        if not r.ok:
            return []

        parser = Parser(r.content)
        nodes = parser.query('//div[@class="title"]/a[contains(@href, "/subtitles/")]')

        movies = []
        processed_urls = set()
        for node in nodes:
            url = node.get('href')
            if url.startswith('/subtitles/release?'):
                continue

            movie_url = urljoin(base_url, url)
            if movie_url in processed_urls:
                continue

            processed_urls.add(movie_url)

            movie_title = node.text.strip()
            m = self.movie_title_year_pattern.search(movie_title)
            movie_year = -1
            if m:
                movie_title = m.group(1).strip()
                if m.group(2):
                    movie_year = int(m.group(3).strip())

            movies.append({
                'title': movie_title,
                'year': movie_year,
                'url': movie_url
            })

        return movies

    def get_movie_subs(self, movie, params, lang):
        movie_url = movie['url']
        # print movie_url
        r = self.session.get(movie_url)
        # print r.content

        m = re.compile('/subtitles/([^/]+)$').search(movie_url)
        movie_slug_name = m.group(1)

        parser = Parser(r.content)
        lang_full_name = get_full_lang(lang)
        nodes = parser.query('//a[contains(@href, "/subtitles/%s/%s/")]' % (movie_slug_name, lang_full_name))

        # pprint(nodes)

        # subtitle_match_str = ' '.join(sorted(params['release_name_tokens']))
        # subtitle_match_tokens = set(params['release_name_tokens'])
        # print 'subtitle_match_str', subtitle_match_str

        subtitles = []
        for node in nodes:
            subtitle_url = node.get('href')
            subtitle_name = node.find('span[2]').text.strip()

            # tmp1 = set(tokenizer(subtitle_name))
            # d = len(subtitle_match_tokens.intersection(tmp1)) * 100 - len(tmp1)

            subtitles.append({
                'url': urljoin(movie_url, subtitle_url),
                'name': subtitle_name,
                # 'd': d
            })

        return subtitles

    def download_sub(self, release, target_folder, release_name):
        sub_page_url = release['url']
        r = self.session.get(sub_page_url)
        if not r.ok:
            # print r.content
            return None

        m = re.search('href=\"(/subtitle/download[^\'"]+)"', r.text)
        if not m:
            # print 'Could not find download url'
            return None

        sub_download_url = urljoin(sub_page_url, m.group(1))

        tmp_folder = mkdtemp(prefix='subfind-')
        try:
            response = requests.get(sub_download_url, stream=True)
            if not response.ok:
                # Something went wrong
                # print 'could not download sub file'
                return None

            file_format = 'zip'
            if 'rar' in response.headers['Content-Type']:
                file_format = 'rar'

            tmp_file = abspath(join(tmp_folder, 'tmp.%s' % file_format))

            with open(tmp_file, 'wb') as handle:
                for block in response.iter_content(1024):
                    handle.write(block)

            if file_format == 'zip':
                with open(tmp_file, 'rb') as fh:
                    z = zipfile.ZipFile(fh)
                    num_items = 0
                    for item in z.namelist():
                        num_items += 1
                        for sub_extension in self.sub_extensions:
                            if item.endswith('.%s' % sub_extension):
                                z.extract(item, tmp_folder)
                                sub_file = join(tmp_folder, item)

                                return self._save_sub(release, sub_file, target_folder, release_name, sub_extension)

                    if num_items == 0:
                        raise SubtitleFileBroken(url=sub_page_url, message='Could not find any file in subtitle file')
            elif file_format == 'rar':
                run_cmd('unrar e -inul %s' % tmp_file, cwd=tmp_folder)
                for item in os.listdir(tmp_folder):
                    ext = get_subtitle_ext(item)
                    if not ext:
                        continue

                    sub_file = join(tmp_folder, item)
                    return self._save_sub(release, sub_file, target_folder, release_name, ext)
            else:
                raise Exception('Un-support extract file extension %s' % file_format)

            self.logger.warning('There is no subtitle extension found. Files:')
            for item in listdir(tmp_folder):
                self.logger.warning(item)

            return None
        finally:
            # Remove tmp_folder
            rmtree(tmp_folder)

    def get_releases(self, release_name, langs):
        release_matching_checker = ReleaseMatchingChecker(release_name)
        # Using title_query to search release will return more releases than using release_name
        query = release_matching_checker.info['title_query']
        base_url = (SUBSCENE_RELEASE_SEARCH_URL % urlencode({'q': query, 'l': '', 'r': 'true'}))

        r = self.session.get(base_url)

        if not r.ok:
            raise HTTPConnectionError(base_url, r.status_code, r.content)

        parser = Parser(r.content)
        release_nodes = parser.query("//td/a[contains(@href, '/subtitles/')]")
        ret = {}
        if not release_nodes:
            return ret

        for release_node in release_nodes:
            release_lang = release_node.find('span[1]').text.strip()
            release_lang = get_short_lang(release_lang)

            if release_lang not in langs:
                continue

            if release_lang not in ret:
                ret[release_lang] = []

            item_release_name = release_node.find('span[2]').text.strip()

            try:
                release_matching_checker.check(item_release_name)
            except ReleaseNotMatchError:
                continue

            release_url = urljoin(base_url, release_node.get('href'))

            release = {
                'name': item_release_name,
                'lang': release_lang,
                'url': release_url
            }

            ret[release_lang].append(release)

        return ret

==> subfind_provider_subscene-3.2.1.0-py2.py3-none-any.whl: subfind_provider_subscene/__init__.py <==
import logging
import zipfile

from six.moves.urllib.parse import urlencode, urljoin

import os
import re
import requests
from os import listdir
from os.path import join, abspath
from shutil import rmtree
from subfind import BaseProvider
from subfind.cmd import run_cmd
from subfind.exception import SubtitleFileBroken, HTTPConnectionError, ReleaseNotMatchError
from subfind.movie.alice import MovieScoringAlice
from subfind.parser import Parser
from subfind.release import ReleaseMatchingChecker
from subfind.release.alice import ReleaseScoringAlice
from subfind.scenario import BaseScenarioFactory, Scenario1
from subfind.utils.subtitle import get_subtitle_ext
from subfind_provider_subscene.language import get_full_lang, get_short_lang
from tempfile import mkdtemp

SUBSCENE_SEARCH_URL = "http://subscene.com/subtitles/title?%s"
SUBSCENE_RELEASE_SEARCH_URL = "http://subscene.com/subtitles/release?%s"


class SubsceneFactory(BaseScenarioFactory):
    def get_scenario(self):
        release_scoring = ReleaseScoringAlice()
        provider = SubsceneProvider()

        return Scenario1(release_scoring, provider)


class SubsceneProvider(BaseProvider):
    def __init__(self):
        self.session = requests.Session()

        self.movie_title_year_pattern = re.compile('^(.*)(\s+\((\d+)\))$')
        self.sub_extensions = ['srt', 'sub', 'ass']
        self.logger = logging.getLogger(self.__class__.__name__)

        self.movie_score = MovieScoringAlice()

    def search_movie(self, release_name, langs):
        release_matching_checker = ReleaseMatchingChecker(release_name)

        query = release_matching_checker.info['title_query']
        base_url = (SUBSCENE_SEARCH_URL % urlencode({'q': query}))

        r = self.session.get(base_url)

        if not r.ok:
            return []

        parser = Parser(r.content)
        nodes = parser.query('//div[@class="title"]/a[contains(@href, "/subtitles/")]')

        movies = []
        processed_urls = set()
        for node in nodes:
            url = node.get('href')
            if url.startswith('/subtitles/release?'):
                continue

            movie_url = urljoin(base_url, url)
            if movie_url in processed_urls:
                continue

            processed_urls.add(movie_url)

            movie_title = node.text.strip()
            m = self.movie_title_year_pattern.search(movie_title)
            movie_year = -1
            if m:
                movie_title = m.group(1).strip()
                if m.group(2):
                    movie_year = int(m.group(3).strip())

            # release = {
            #     'name': item_release_name,
            #     'lang': release_lang,
            #     'url': release_url
            # }

            movies.append({
                'title': movie_title,
                'year': movie_year,
                'url': movie_url
            })

        if not movies:
            return movies

        self.movie_score.sort(release_matching_checker.info, movies)

        movie = movies[0]
        return self._get_movie_release(release_matching_checker, movie, langs)

        return movies

    def _get_movie_release(self, release_matching_checker, movie, langs):
        base_url = movie['url']
        r = self.session.get(base_url)

        if not r.ok:
            raise HTTPConnectionError(base_url, r.status_code, r.content)

        parser = Parser(r.content)
        release_nodes = parser.query("//td/a[contains(@href, '/subtitles/')]")
        ret = {}
        if not release_nodes:
            return ret

        for release_node in release_nodes:
            release_lang = release_node.find('span[1]').text.strip()
            release_lang = get_short_lang(release_lang)

            if release_lang not in langs:
                continue

            if release_lang not in ret:
                ret[release_lang] = []

            item_release_name = release_node.find('span[2]').text.strip()

            try:
                release_matching_checker.check(item_release_name)
            except ReleaseNotMatchError:
                continue

            release_url = urljoin(base_url, release_node.get('href'))

            release = {
                'name': item_release_name,
                'lang': release_lang,
                'url': release_url
            }

            ret[release_lang].append(release)

        return ret

    def get_movie_subs(self, movie, params, lang):
        movie_url = movie['url']
        # print movie_url
        r = self.session.get(movie_url)
        # print r.content

        m = re.compile('/subtitles/([^/]+)$').search(movie_url)
        movie_slug_name = m.group(1)

        parser = Parser(r.content)
        lang_full_name = get_full_lang(lang)
        nodes = parser.query('//a[contains(@href, "/subtitles/%s/%s/")]' % (movie_slug_name, lang_full_name))

        # pprint(nodes)

        # subtitle_match_str = ' '.join(sorted(params['release_name_tokens']))
        # subtitle_match_tokens = set(params['release_name_tokens'])
        # print 'subtitle_match_str', subtitle_match_str

        subtitles = []
        for node in nodes:
            subtitle_url = node.get('href')
            subtitle_name = node.find('span[2]').text.strip()

            # tmp1 = set(tokenizer(subtitle_name))
            # d = len(subtitle_match_tokens.intersection(tmp1)) * 100 - len(tmp1)

            subtitles.append({
                'url': urljoin(movie_url, subtitle_url),
                'name': subtitle_name,
                # 'd': d
            })

        return subtitles

    def download_sub(self, release, target_folder, release_name):
        sub_page_url = release['url']
        r = self.session.get(sub_page_url)
        if not r.ok:
            # print r.content
            return None

        m = re.search('href=\"(/subtitle/download[^\'"]+)"', r.text)
        if not m:
            # print 'Could not find download url'
            return None

        sub_download_url = urljoin(sub_page_url, m.group(1))

        tmp_folder = mkdtemp(prefix='subfind-')
        try:
            response = requests.get(sub_download_url, stream=True)
            if not response.ok:
                # Something went wrong
                # print 'could not download sub file'
                return None

            file_format = 'zip'
            if 'rar' in response.headers['Content-Type']:
                file_format = 'rar'

            tmp_file = abspath(join(tmp_folder, 'tmp.%s' % file_format))

            with open(tmp_file, 'wb') as handle:
                for block in response.iter_content(1024):
                    handle.write(block)

            if file_format == 'zip':
                with open(tmp_file, 'rb') as fh:
                    z = zipfile.ZipFile(fh)
                    num_items = 0
                    for item in z.namelist():
                        num_items += 1
                        for sub_extension in self.sub_extensions:
                            if item.endswith('.%s' % sub_extension):
                                z.extract(item, tmp_folder)
                                sub_file = join(tmp_folder, item)

                                return self._save_sub(release, sub_file, target_folder, release_name, sub_extension)

                    if num_items == 0:
                        raise SubtitleFileBroken(url=sub_page_url, message='Could not find any file in subtitle file')
            elif file_format == 'rar':
                run_cmd('unrar e -inul %s' % tmp_file, cwd=tmp_folder)
                for item in os.listdir(tmp_folder):
                    ext = get_subtitle_ext(item)
                    if not ext:
                        continue

                    sub_file = join(tmp_folder, item)
                    return self._save_sub(release, sub_file, target_folder, release_name, ext)
            else:
                raise Exception('Un-support extract file extension %s' % file_format)

            self.logger.warning('There is no subtitle extension found. Files:')
            for item in listdir(tmp_folder):
                self.logger.warning(item)

            return None
        finally:
            # Remove tmp_folder
            rmtree(tmp_folder)

    def get_releases(self, release_name, langs):
        return self.search_movie(release_name, langs)

    def get_releases_old(self, release_name, langs):
        """
        Search release is no longer support by subscene
        :param release_name:
        :type release_name:
        :param langs:
        :type langs:
        :return:
        :rtype:
        """
        release_matching_checker = ReleaseMatchingChecker(release_name)
        # Using title_query to search release will return more releases than using release_name
        query = release_matching_checker.info['title_query']
        base_url = (SUBSCENE_RELEASE_SEARCH_URL % urlencode({'q': query, 'l': '', 'r': 'true'}))

        r = self.session.get(base_url)

        if not r.ok:
            raise HTTPConnectionError(base_url, r.status_code, r.content)

        parser = Parser(r.content)
        release_nodes = parser.query("//td/a[contains(@href, '/subtitles/')]")
        ret = {}
        if not release_nodes:
            return ret

        for release_node in release_nodes:
            release_lang = release_node.find('span[1]').text.strip()
            release_lang = get_short_lang(release_lang)

            if release_lang not in langs:
                continue

            if release_lang not in ret:
                ret[release_lang] = []

            item_release_name = release_node.find('span[2]').text.strip()

            try:
                release_matching_checker.check(item_release_name)
            except ReleaseNotMatchError:
                continue

            release_url = urljoin(base_url, release_node.get('href'))

            release = {
                'name': item_release_name,
                'lang': release_lang,
                'url': release_url
            }

            ret[release_lang].append(release)

        return ret
